package geniusweb.protocol;

import java.util.Collections;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonTypeInfo;

import geniusweb.issuevalue.Bid;
import geniusweb.references.PartyWithProfile;

/**
 * All results collected from running a session. Normally this does not contain
 * all the details, only the final outcome and other global notes.
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE)
public class SessionResult {
	private final List<PartyWithProfile> participants;
	private final Bid agreement;

	// add more type info so that jackson can deserialize the actual class
	@JsonTypeInfo(use = JsonTypeInfo.Id.CLASS, include = JsonTypeInfo.As.WRAPPER_OBJECT)
	private final Throwable error;

	/**
	 * 
	 * @param participants the list oof {@link PartyWithProfile}, in the proper
	 *                     order (this is relevant for the protocol etc). Should
	 *                     never be null. Some of them may have entered later of
	 *                     left early. This list should contain them all.
	 * @param agreement    the final agreement {@link Bid} of the session. Null
	 *                     if no agreement was reached.
	 * @param error        a fatal error that terminated the session. Non-fatal
	 *                     errors (warnings) are not to be reported. Null if no
	 *                     fatal error occured (session ended following the
	 *                     protocol). Normally, a fatal error results in no
	 *                     agreement but there might be protocols that allow a
	 *                     session to end with an error but stick with a already
	 *                     reached agreement as well.
	 */
	@JsonCreator
	public SessionResult(
			@JsonProperty("participants") List<PartyWithProfile> participants,
			@JsonProperty("agreement") Bid agreement,
			@JsonProperty("error") Throwable error) {
		this.participants = participants;
		this.agreement = agreement;
		this.error = error;
	}

	/**
	 * 
	 * @return the list oof {@link PartyWithProfile}, in the proper order (this
	 *         is relevant for the protocol etc). Should never be null. Some of
	 *         them may have entered later of left early. This list should
	 *         contain them all.
	 */
	public List<PartyWithProfile> getParticipants() {
		return Collections.unmodifiableList(participants);
	}

	/**
	 * 8return the final agreement {@link Bid} of the session. Null if no
	 * agreement was reached.
	 */
	public Bid getAgreement() {
		return agreement;
	};

	/**
	 * 
	 * @return a fatal error that terminated the session. Non-fatal errors
	 *         (warnings) are not to be reported. Null if no fatal error occured
	 *         (session ended following the protocol). Normally, a fatal error
	 *         results in no agreement but there might be protocols that allow a
	 *         session to end with an error but stick with a already reached
	 *         agreement as well.
	 */
	public Throwable getError() {
		return error;
	}

	@Override
	public String toString() {
		return "SessionResult[" + participants + "," + agreement + "," + error
				+ "]";
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((agreement == null) ? 0 : agreement.hashCode());
		result = prime * result + ((error == null) ? 0 : error.hashCode());
		result = prime * result
				+ ((participants == null) ? 0 : participants.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		SessionResult other = (SessionResult) obj;
		if (agreement == null) {
			if (other.agreement != null)
				return false;
		} else if (!agreement.equals(other.agreement))
			return false;
		if (error == null) {
			if (other.error != null)
				return false;
		} else if (other.error == null)
			return false;
		else if (!error.toString().equals(other.error.toString()))
			// HACK error does not have equals implemented!
			// so we hacked in toString() here to work around it.
			return false;
		if (participants == null) {
			if (other.participants != null)
				return false;
		} else if (!participants.equals(other.participants))
			return false;
		return true;
	}

}
